@@ -5,28 +5,28 @@ import logging
 import os
 import urllib3.util.connection
 import random
-import requests
 import six
 import socket
 import time
 
 from dns.exception import DNSException
 from dns import resolver
-from patroni.dcs import AbstractDCS, ClusterConfig, Cluster, Failover, Leader, Member, SyncState, TimelineHistory
-from patroni.exceptions import DCSError
-from patroni.utils import Retry, RetryFailedError, split_host_port
-from urllib3.exceptions import HTTPError, ReadTimeoutError
-from requests.exceptions import RequestException
+from urllib3.exceptions import HTTPError, ReadTimeoutError, ProtocolError
 from six.moves.queue import Queue
 from six.moves.http_client import HTTPException
 from six.moves.urllib_parse import urlparse
 from threading import Thread
 
+from . import AbstractDCS, Cluster, ClusterConfig, Failover, Leader, Member, SyncState, TimelineHistory
+from ..exceptions import DCSError
+from ..request import get as requests_get
+from ..utils import Retry, RetryFailedError, split_host_port, uri, USER_AGENT
+
 logger = logging.getLogger(__name__)
 
 
-def uri(protocol, host, port, endpoint=''):
-    return '{0}://{1}:{2}{3}'.format(protocol, host, port, endpoint)
+class EtcdRaftInternal(etcd.EtcdException):
+    """Raft Internal Error"""
 
 
 class EtcdError(DCSError):
@@ -73,8 +73,8 @@ class DnsCachingResolver(Thread):
     def _do_resolve(host, port):
         try:
             return socket.getaddrinfo(host, port, 0, socket.SOCK_STREAM, socket.IPPROTO_TCP)
-        except socket.gaierror:
-            logger.warning('failed to resolve host %s', host)
+        except Exception as e:
+            logger.warning('failed to resolve host %s: %s', host, e)
             return []
 
 
@@ -87,25 +87,57 @@ class Client(etcd.Client):
         args = {p: config.get(p) for p in ('host', 'port', 'protocol', 'use_proxies', 'username', 'password',
                                            'cert', 'ca_cert') if config.get(p)}
         super(Client, self).__init__(read_timeout=config['retry_timeout'], **args)
+        # For some reason python3-etcd on debian and ubuntu are not based on the latest version
+        # Workaround for the case when https://github.com/jplana/python-etcd/pull/196 is not applied
+        self.http.connection_pool_kw.pop('ssl_version', None)
         self._config = config
         self._load_machines_cache()
-        self._allow_reconnect = not self._use_proxies
+        self._allow_reconnect = True
+        # allow passing retry argument to api_execute in params
+        self._comparison_conditions.add('retry')
+        self._read_options.add('retry')
+        self._del_conditions.add('retry')
 
-    def _build_request_parameters(self):
+    def _calculate_timeouts(self, etcd_nodes=None, timeout=None):
+        """Calculate a request timeout and number of retries per single etcd node.
+        In case if the timeout per node is too small (less than one second) we will reduce the number of nodes.
+        For the cluster with only one node we will try to do 2 retries.
+        For clusters with 2 nodes we will try to do 1 retry for every node.
+        No retries for clusters with 3 or more nodes. We better rely on switching to a different node."""
+
+        etcd_nodes = etcd_nodes or len(self._machines_cache) + 1
+        per_node_timeout = timeout = float(timeout or self.read_timeout)
+
+        max_retries = 4 - min(etcd_nodes, 3)
+        per_node_retries = 1
+        min_timeout = 1.0
+
+        while etcd_nodes > 0:
+            per_node_timeout = float(timeout) / etcd_nodes
+            if per_node_timeout >= min_timeout:
+                # for small clusters we will try to do more than on try on every node
+                while per_node_retries < max_retries and per_node_timeout / (per_node_retries + 1) >= min_timeout:
+                    per_node_retries += 1
+                per_node_timeout /= per_node_retries
+                break
+            # if the timeout per one node is to small try to reduce number of nodes
+            etcd_nodes -= 1
+            max_retries = 1
+
+        return etcd_nodes, per_node_timeout, per_node_retries - 1
+
+    def _get_headers(self):
+        basic_auth = ':'.join((self.username, self.password)) if self.username and self.password else None
+        return urllib3.make_headers(basic_auth=basic_auth, user_agent=USER_AGENT)
+
+    def _build_request_parameters(self, timeout=None):
         kwargs = {'headers': self._get_headers(), 'redirect': self.allow_redirect}
 
-        # calculate the number of retries and timeout *per node*
-        # actual number of retries depends on the number of nodes
-        etcd_nodes = len(self._machines_cache) + 1
-        kwargs['retries'] = 0 if etcd_nodes > 3 else (1 if etcd_nodes > 1 else 2)
-
-        # if etcd_nodes > 3:
-        #     kwargs.update({'retries': 0, 'timeout': float(self.read_timeout)/etcd_nodes})
-        # elif etcd_nodes > 1:
-        #     kwargs.update({'retries': 1, 'timeout': self.read_timeout/2.0/etcd_nodes})
-        # else:
-        #     kwargs.update({'retries': 2, 'timeout': self.read_timeout/3.0})
-        kwargs['timeout'] = self.read_timeout/float(kwargs['retries'] + 1)/etcd_nodes
+        if timeout is not None:
+            kwargs.update(retries=0, timeout=timeout)
+        else:
+            _, per_node_timeout, per_node_retries = self._calculate_timeouts()
+            kwargs.update(timeout=per_node_timeout, retries=per_node_retries)
         return kwargs
 
     def set_machines_cache_ttl(self, cache_ttl):
@@ -128,8 +160,11 @@ class Client(etcd.Client):
         while True:
             try:
                 response = self.http.request(self._MGET, self._base_uri + self.version_prefix + '/machines', **kwargs)
-                machines = [n.strip() for n in self._handle_server_response(response).data.decode('utf-8').split(',')]
+                data = self._handle_server_response(response).data.decode('utf-8')
+                machines = [m.strip() for m in data.split(',') if m.strip()]
                 logger.debug("Retrieved list of machines: %s", machines)
+                if not machines:
+                    raise etcd.EtcdException
                 random.shuffle(machines)
                 for url in machines:
                     r = urlparse(url)
@@ -160,8 +195,12 @@ class Client(etcd.Client):
             self._check_cluster_id(response)
         except (HTTPError, HTTPException, socket.error, socket.timeout) as e:
             if (isinstance(fields, dict) and fields.get("wait") == "true" and
-                    isinstance(e, ReadTimeoutError)):
+                    isinstance(e, (ReadTimeoutError, ProtocolError))):
                 logger.debug("Watch timed out.")
+                # switch to the next etcd node because we don't know exactly what happened,
+                # whether the key didn't received an update or there is a network problem.
+                self._machines_cache.insert(0, self._base_uri)
+                self._base_uri = self._next_server()
                 raise etcd.EtcdWatchTimedOut("Watch timed out: {0}".format(e), cause=e)
             logger.error("Request to server %s failed: %r", self._base_uri, e)
             logger.info("Reconnection allowed, looking for another server.")
@@ -173,6 +212,7 @@ class Client(etcd.Client):
         if not path.startswith('/'):
             raise ValueError('Path does not start with /')
 
+        retry = params.pop('retry', None) if isinstance(params, dict) else None
         kwargs = {'fields': params, 'preload_content': False}
 
         if method in [self._MGET, self._MDELETE]:
@@ -186,37 +226,46 @@ class Client(etcd.Client):
         # Update machines_cache if previous attempt of update has failed
         if self._update_machines_cache:
             self._load_machines_cache()
-        elif time.time() - self._machines_cache_updated > self._machines_cache_ttl:
-            self._machines_cache = self.machines
-            if self._base_uri in self._machines_cache:
-                self._machines_cache.remove(self._base_uri)
-            self._machines_cache_updated = time.time()
+        elif not self._use_proxies and time.time() - self._machines_cache_updated > self._machines_cache_ttl:
+            self._refresh_machines_cache()
 
-        kwargs.update(self._build_request_parameters())
+        kwargs.update(self._build_request_parameters(timeout))
 
-        if timeout is not None:
-            kwargs.update({'retries': 0, 'timeout': timeout})
+        if retry:
+            machines_cache = [self._base_uri] + self._machines_cache
 
         response = False
 
-        try:
-            some_request_failed = False
-            while not response:
-                response = self._do_http_request(request_executor, method, self._base_uri + path, **kwargs)
+        while True:
+            try:
+                some_request_failed = False
+                while not response:
+                    response = self._do_http_request(request_executor, method, self._base_uri + path, **kwargs)
+
+                    if response is False:
+                        if not retry:
+                            raise etcd.EtcdException('{0} {1} request failed'.format(method, path))
+                        some_request_failed = True
+                if some_request_failed:
+                    self._refresh_machines_cache()
+                if response:
+                    break
+            except etcd.EtcdConnectionFailed:
+                if not retry:
+                    raise
+                sleeptime = retry.sleeptime
+                remaining_time = retry.stoptime - sleeptime - time.time()
+                nodes, timeout, retries = self._calculate_timeouts(len(machines_cache), remaining_time)
+                if nodes == 0:
+                    self._update_machines_cache = True
+                    raise
+                retry.sleep_func(sleeptime)
+                retry.update_delay()
+                # We still have some time left. Partially restore `_machines_cache` and retry request
+                kwargs.update(timeout=timeout, retries=retries)
+                self._base_uri = machines_cache[0]
+                self._machines_cache = machines_cache[1:nodes]
 
-                if response is False:
-                    some_request_failed = True
-            if some_request_failed and not self._use_proxies:
-                self._machines_cache = self.machines
-                if self._base_uri in self._machines_cache:
-                    self._machines_cache.remove(self._base_uri)
-        except etcd.EtcdConnectionFailed as e:
-            if isinstance(e, etcd.EtcdWatchTimedOut) and self._machines_cache:
-                self._base_uri = self._next_server()
-            else:
-                self._update_machines_cache = True
-            if not response:
-                raise
         return self._handle_server_response(response)
 
     @staticmethod
@@ -236,15 +285,15 @@ class Client(etcd.Client):
             protocol = 'https' if '-ssl' in r else 'http'
             endpoint = '/members' if '-server' in r else ''
             for host, port in self.get_srv_record('_etcd{0}._tcp.{1}'.format(r, srv)):
-                url = uri(protocol, host, port, endpoint)
+                url = uri(protocol, (host, port), endpoint)
                 if endpoint:
                     try:
-                        response = requests.get(url, timeout=self.read_timeout, verify=False)
-                        if response.ok:
-                            for member in response.json():
+                        response = requests_get(url, timeout=self.read_timeout, verify=False)
+                        if response.status < 400:
+                            for member in json.loads(response.data.decode('utf-8')):
                                 ret.extend(member['clientURLs'])
                             break
-                    except RequestException:
+                    except Exception:
                         logger.exception('GET %s', url)
                 else:
                     ret.append(url)
@@ -258,15 +307,25 @@ class Client(etcd.Client):
     def _get_machines_cache_from_dns(self, host, port):
         """One host might be resolved into multiple ip addresses. We will make list out of it"""
         if self.protocol == 'http':
-            ret = []
-            for af, _, _, _, sa in self._dns_resolver.resolve(host, port):
-                host, port = sa[:2]
-                if af == socket.AF_INET6:
-                    host = '[{0}]'.format(host)
-                ret.append(uri(self.protocol, host, port))
+            ret = map(lambda res: uri(self.protocol, res[-1][:2]), self._dns_resolver.resolve(host, port))
             if ret:
                 return list(set(ret))
-        return [uri(self.protocol, host, port)]
+        return [uri(self.protocol, (host, port))]
+
+    def _get_machines_cache_from_config(self):
+        if 'proxy' in self._config:
+            return [uri(self.protocol, (self._config['host'], self._config['port']))]
+
+        machines_cache = []
+        if 'srv' in self._config:
+            machines_cache = self._get_machines_cache_from_srv(self._config['srv'])
+
+        if not machines_cache and 'hosts' in self._config:
+            machines_cache = list(self._config['hosts'])
+
+        if not machines_cache and 'host' in self._config:
+            machines_cache = self._get_machines_cache_from_dns(self._config['host'], self._config['port'])
+        return machines_cache
 
     def _load_machines_cache(self):
         """This method should fill up `_machines_cache` from scratch.
@@ -279,19 +338,7 @@ class Client(etcd.Client):
         if 'srv' not in self._config and 'host' not in self._config and 'hosts' not in self._config:
             raise Exception('Neither srv, hosts, host nor url are defined in etcd section of config')
 
-        if self._use_proxies:
-            self._machines_cache = [uri(self.protocol, self._config['host'], self._config['port'])]
-        else:
-            self._machines_cache = []
-
-            if 'srv' in self._config:
-                self._machines_cache = self._get_machines_cache_from_srv(self._config['srv'])
-
-            if not self._machines_cache and 'hosts' in self._config:
-                self._machines_cache = list(self._config['hosts'])
-
-            if not self._machines_cache and 'host' in self._config:
-                self._machines_cache = self._get_machines_cache_from_dns(self._config['host'], self._config['port'])
+        self._machines_cache = self._get_machines_cache_from_config()
 
         # Can not bootstrap list of etcd-cluster members, giving up
         if not self._machines_cache:
@@ -299,12 +346,15 @@ class Client(etcd.Client):
 
         # After filling up initial list of machines_cache we should ask etcd-cluster about actual list
         self._base_uri = self._next_server()
-        self._machines_cache = self.machines
+        self._refresh_machines_cache()
+        self._update_machines_cache = False
 
+    def _refresh_machines_cache(self):
+        self._machines_cache = self._get_machines_cache_from_config() if self._use_proxies else self.machines
         if self._base_uri in self._machines_cache:
             self._machines_cache.remove(self._base_uri)
-
-        self._update_machines_cache = False
+        elif self._machines_cache:
+            self._base_uri = self._next_server()
         self._machines_cache_updated = time.time()
 
 
@@ -314,15 +364,15 @@ class Etcd(AbstractDCS):
         super(Etcd, self).__init__(config)
         self._ttl = int(config.get('ttl') or 30)
         self._retry = Retry(deadline=config['retry_timeout'], max_delay=1, max_tries=-1,
-                            retry_exceptions=(etcd.EtcdLeaderElectionInProgress,
-                                              etcd.EtcdWatcherCleared,
-                                              etcd.EtcdEventIndexCleared))
+                            retry_exceptions=(etcd.EtcdLeaderElectionInProgress, EtcdRaftInternal))
         self._client = self.get_etcd_client(config)
         self.__do_not_watch = False
         self._has_failed = False
 
     def retry(self, *args, **kwargs):
-        return self._retry.copy()(*args, **kwargs)
+        retry = self._retry.copy()
+        kwargs['retry'] = retry
+        return retry(*args, **kwargs)
 
     def _handle_exception(self, e, name='', do_sleep=False, raise_ex=None):
         if not self._has_failed:
@@ -370,7 +420,7 @@ class Etcd(AbstractDCS):
             config['hosts'] = []
             for value in hosts:
                 if isinstance(value, six.string_types):
-                    config['hosts'].append(uri(protocol, *split_host_port(value, default_port)))
+                    config['hosts'].append(uri(protocol, split_host_port(value.strip(), default_port)))
         elif 'host' in config:
             host, port = split_host_port(config['host'], 2379)
             config['host'] = host
@@ -426,6 +476,8 @@ class Etcd(AbstractDCS):
         while not client:
             try:
                 client = Client(config, dns_resolver)
+                if 'use_proxies' in config and not client.machines:
+                    raise etcd.EtcdException
             except etcd.EtcdException:
                 logger.info('waiting on etcd')
                 time.sleep(5)
@@ -437,6 +489,10 @@ class Etcd(AbstractDCS):
         self._ttl = ttl
         self._client.set_machines_cache_ttl(ttl*10)
 
+    @property
+    def ttl(self):
+        return self._ttl
+
     def set_retry_timeout(self, retry_timeout):
         self._retry.deadline = retry_timeout
         self._client.set_read_timeout(retry_timeout)
@@ -446,9 +502,10 @@ class Etcd(AbstractDCS):
         return Member.from_node(node.modifiedIndex, os.path.basename(node.key), node.ttl, node.value)
 
     def _load_cluster(self):
+        cluster = None
         try:
             result = self.retry(self._client.read, self.client_path(''), recursive=True)
-            nodes = {os.path.relpath(node.key, result.key).replace('\\', '/'): node for node in result.leaves}
+            nodes = {node.key[len(result.key):].lstrip('/'): node for node in result.leaves}
 
             # get initialize flag
             initialize = nodes.get(self._INITIALIZE)
@@ -486,21 +543,22 @@ class Etcd(AbstractDCS):
             sync = nodes.get(self._SYNC)
             sync = SyncState.from_node(sync and sync.modifiedIndex, sync and sync.value)
 
-            self._cluster = Cluster(initialize, config, leader, last_leader_operation, members, failover, sync, history)
+            cluster = Cluster(initialize, config, leader, last_leader_operation, members, failover, sync, history)
         except etcd.EtcdKeyNotFound:
-            self._cluster = Cluster(None, None, None, None, [], None, None, None)
+            cluster = Cluster(None, None, None, None, [], None, None, None)
         except Exception as e:
             self._handle_exception(e, 'get_cluster', raise_ex=EtcdError('Etcd is not responding properly'))
         self._has_failed = False
+        return cluster
 
     @catch_etcd_errors
-    def touch_member(self, data, ttl=None, permanent=False):
+    def touch_member(self, data, permanent=False):
         data = json.dumps(data, separators=(',', ':'))
-        return self._client.set(self.member_path, data, None if permanent else ttl or self._ttl)
+        return self._client.set(self.member_path, data, None if permanent else self._ttl)
 
     @catch_etcd_errors
     def take_leader(self):
-        return self.retry(self._client.set, self.leader_path, self._name, self._ttl)
+        return self.retry(self._client.write, self.leader_path, self._name, ttl=self._ttl)
 
     def attempt_to_acquire_leader(self, permanent=False):
         try:
@@ -529,7 +587,7 @@ class Etcd(AbstractDCS):
 
     @catch_etcd_errors
     def _update_leader(self):
-        return self.retry(self._client.test_and_set, self.leader_path, self._name, self._name, self._ttl)
+        return self.retry(self._client.write, self.leader_path, self._name, prevValue=self._name, ttl=self._ttl)
 
     @catch_etcd_errors
     def initialize(self, create_new=True, sysid=""):
@@ -590,3 +648,6 @@ class Etcd(AbstractDCS):
             return super(Etcd, self).watch(None, timeout)
         finally:
             self.event.clear()
+
+
+etcd.EtcdError.error_exceptions[300] = EtcdRaftInternal
